@@ -24,7 +24,7 @@ debian_prepare()
     echo
     echo Profanity installer... installing dependencies
     echo
-    sudo apt-get -y install git automake autoconf libssl-dev libexpat1-dev libncursesw5-dev libglib2.0-dev libnotify-dev libcurl3-dev libxss-dev libotr5-dev libtool
+    sudo apt-get -y install git automake autoconf libssl-dev libexpat1-dev libncursesw5-dev libglib2.0-dev libnotify-dev libcurl3-dev libxss-dev libotr5-dev libreadline-dev libtool libgpgme11-dev uuid-dev
 
 }
 
@@ -34,9 +34,7 @@ fedora_prepare()
     echo Profanity installer... installing dependencies
     echo
 
-    ARCH=`arch`
-
-    sudo yum -y install gcc git autoconf automake openssl-devel.$ARCH expat-devel.$ARCH ncurses-devel.$ARCH  glib2-devel.$ARCH libnotify-devel.$ARCH libcurl-devel.$ARCH libXScrnSaver-devel.$ARCH libotr3-devel.$ARCH libtool
+    sudo dnf -y install gcc git autoconf automake openssl-devel expat-devel ncurses-devel glib2-devel libnotify-devel libcurl-devel libXScrnSaver-devel libotr3-devel readline-devel libtool libuuid-devel gpgme-devel
 }
 
 opensuse_prepare()
@@ -44,7 +42,19 @@ opensuse_prepare()
     echo
     echo Profanity installer...installing dependencies
     echo
-    sudo zypper -n in gcc git automake make autoconf libopenssl-devel expat libexpat-devel ncurses-devel glib2-devel libnotify-devel libcurl-devel libXScrnSaver-devel libotr-devel libtool
+    sudo zypper -n in gcc git automake make autoconf libopenssl-devel expat libexpat-devel ncurses-devel glib2-devel libnotify-devel libcurl-devel libXScrnSaver-devel libotr-devel readline-devel libtool libuuid-devel libgpgme-devel
+}
+
+centos_prepare()
+{
+    echo
+    echo Profanity installer...installing dependencies
+    echo
+
+    sudo yum -y install epel-release
+    sudo yum -y install git
+    sudo yum -y install gcc autoconf automake cmake
+    sudo yum -y install openssl-devel expat-devel ncurses-devel glib2-devel libnotify-devel libcurl-devel libXScrnSaver-devel libotr-devel readline-devel libtool libuuid-devel gpgme-devel
 }
 
 cygwin_prepare()
@@ -60,9 +70,9 @@ cygwin_prepare()
     mv apt-cyg /usr/local/bin/
 
     if [ -n "$CYG_MIRROR" ]; then
-        apt-cyg -m $CYG_MIRROR install git make gcc-core m4 automake autoconf pkg-config openssl-devel libexpat-devel zlib-devel libncursesw-devel libglib2.0-devel libcurl-devel libidn-devel libssh2-devel libkrb5-devel openldap-devel libgcrypt-devel libtool
+        apt-cyg -m $CYG_MIRROR install git make gcc-core m4 automake autoconf pkg-config openssl-devel libexpat-devel zlib-devel libncursesw-devel libglib2.0-devel libcurl-devel libidn-devel libssh2-devel libkrb5-devel openldap-devel libgcrypt-devel libreadline-devel libgpgme-devel libtool libuuid-devel
     else
-        apt-cyg install git make gcc-core m4 automake autoconf pkg-config openssl-devel libexpat-devel zlib-devel libncursesw-devel libglib2.0-devel libcurl-devel libidn-devel libssh2-devel libkrb5-devel openldap-devel libgcrypt-devel libtool
+        apt-cyg install git make gcc-core m4 automake autoconf pkg-config openssl-devel libexpat-devel zlib-devel libncursesw-devel libglib2.0-devel libcurl-devel libidn-devel libssh2-devel libkrb5-devel openldap-devel libgcrypt-devel libreadline-devel libgpgme-devel libtool libuuid-devel
 
     fi
 }
@@ -74,7 +84,7 @@ install_lib_strophe()
     echo
     git clone git://github.com/strophe/libstrophe.git
     cd libstrophe
-    git checkout 0.8.7
+    git checkout 0.8.8
     ./bootstrap.sh
     ./configure --prefix=$1
     make
@@ -103,7 +113,7 @@ cyg_install_lib_strophe()
     echo
     git clone git://github.com/strophe/libstrophe.git
     cd libstrophe
-    git checkout 0.8.7
+    git checkout 0.8.8
     ./bootstrap.sh
     ./bootstrap.sh # second call seems to fix problem on cygwin
     ./configure --prefix=/usr
@@ -157,6 +167,8 @@ if [ "${OS}" = "Linux" ]; then
         DIST=fedora
     elif [ -f /etc/debian_version ]; then
         DIST=debian
+    elif [ -f /etc/centos-release ]; then
+        DIST=centos
     elif [ -f /etc/os-release ]; then
         DIST=opensuse
     fi
@@ -185,6 +197,11 @@ opensuse)   opensuse_prepare
             sudo /sbin/ldconfig
             install_profanity
             ;;
+centos)     centos_prepare
+            install_lib_strophe /usr
+            sudo ldconfig
+            install_profanity
+            ;;
 cygwin)     cygwin_prepare
             cyg_install_lib_strophe
             cyg_install_profanity
