@@ -121,6 +121,7 @@ static struct colours_t {
         NCURSES_COLOR_T otruntrusted;
         NCURSES_COLOR_T rosterheader;
         NCURSES_COLOR_T occupantsheader;
+        NCURSES_COLOR_T receiptsent;
 } colour_prefs;
 
 static NCURSES_COLOR_T _lookup_colour(const char * const colour);
@@ -159,7 +160,7 @@ _theme_load_file(const char * const theme_name)
 {
     // use default theme
     if (theme_name == NULL || strcmp(theme_name, "default") == 0) {
-        if (theme != NULL) {
+        if (theme) {
             g_key_file_free(theme);
         }
         theme = g_key_file_new();
@@ -172,12 +173,12 @@ _theme_load_file(const char * const theme_name)
             return FALSE;
         }
 
-        if (theme_loc != NULL) {
+        if (theme_loc) {
             g_string_free(theme_loc, TRUE);
         }
         theme_loc = new_theme_file;
         log_info("Loading theme \"%s\"", theme_name);
-        if (theme != NULL) {
+        if (theme) {
             g_key_file_free(theme);
         }
         theme = g_key_file_new();
@@ -204,14 +205,17 @@ theme_list(void)
 void
 theme_close(void)
 {
-    if (theme != NULL) {
+    if (theme) {
         g_key_file_free(theme);
+        theme = NULL;
     }
-    if (theme_loc != NULL) {
+    if (theme_loc) {
         g_string_free(theme_loc, TRUE);
+        theme_loc = NULL;
     }
     if (bold_items) {
         g_hash_table_destroy(bold_items);
+        bold_items = NULL;
     }
 }
 
@@ -251,47 +255,48 @@ theme_init_colours(void)
     // chat
     init_pair(25, colour_prefs.me, colour_prefs.bkgnd);
     init_pair(26, colour_prefs.them, colour_prefs.bkgnd);
+    init_pair(27, colour_prefs.receiptsent, colour_prefs.bkgnd);
 
     // room chat
-    init_pair(27, colour_prefs.roominfo, colour_prefs.bkgnd);
-    init_pair(28, colour_prefs.roommention, colour_prefs.bkgnd);
+    init_pair(28, colour_prefs.roominfo, colour_prefs.bkgnd);
+    init_pair(29, colour_prefs.roommention, colour_prefs.bkgnd);
 
     // statuses
-    init_pair(29, colour_prefs.online, colour_prefs.bkgnd);
-    init_pair(30, colour_prefs.offline, colour_prefs.bkgnd);
-    init_pair(31, colour_prefs.away, colour_prefs.bkgnd);
-    init_pair(32, colour_prefs.chat, colour_prefs.bkgnd);
-    init_pair(33, colour_prefs.dnd, colour_prefs.bkgnd);
-    init_pair(34, colour_prefs.xa, colour_prefs.bkgnd);
+    init_pair(30, colour_prefs.online, colour_prefs.bkgnd);
+    init_pair(31, colour_prefs.offline, colour_prefs.bkgnd);
+    init_pair(32, colour_prefs.away, colour_prefs.bkgnd);
+    init_pair(33, colour_prefs.chat, colour_prefs.bkgnd);
+    init_pair(34, colour_prefs.dnd, colour_prefs.bkgnd);
+    init_pair(35, colour_prefs.xa, colour_prefs.bkgnd);
 
     // states
-    init_pair(35, colour_prefs.typing, colour_prefs.bkgnd);
-    init_pair(36, colour_prefs.gone, colour_prefs.bkgnd);
+    init_pair(36, colour_prefs.typing, colour_prefs.bkgnd);
+    init_pair(37, colour_prefs.gone, colour_prefs.bkgnd);
 
     // subscription status
-    init_pair(37, colour_prefs.subscribed, colour_prefs.bkgnd);
-    init_pair(38, colour_prefs.unsubscribed, colour_prefs.bkgnd);
+    init_pair(38, colour_prefs.subscribed, colour_prefs.bkgnd);
+    init_pair(39, colour_prefs.unsubscribed, colour_prefs.bkgnd);
 
     // otr messages
-    init_pair(39, colour_prefs.otrstartedtrusted, colour_prefs.bkgnd);
-    init_pair(40, colour_prefs.otrstarteduntrusted, colour_prefs.bkgnd);
-    init_pair(41, colour_prefs.otrended, colour_prefs.bkgnd);
-    init_pair(42, colour_prefs.otrtrusted, colour_prefs.bkgnd);
-    init_pair(43, colour_prefs.otruntrusted, colour_prefs.bkgnd);
+    init_pair(40, colour_prefs.otrstartedtrusted, colour_prefs.bkgnd);
+    init_pair(41, colour_prefs.otrstarteduntrusted, colour_prefs.bkgnd);
+    init_pair(42, colour_prefs.otrended, colour_prefs.bkgnd);
+    init_pair(43, colour_prefs.otrtrusted, colour_prefs.bkgnd);
+    init_pair(44, colour_prefs.otruntrusted, colour_prefs.bkgnd);
 
     // subwin headers
-    init_pair(44, colour_prefs.rosterheader, colour_prefs.bkgnd);
-    init_pair(45, colour_prefs.occupantsheader, colour_prefs.bkgnd);
+    init_pair(45, colour_prefs.rosterheader, colour_prefs.bkgnd);
+    init_pair(46, colour_prefs.occupantsheader, colour_prefs.bkgnd);
 
     // raw
-    init_pair(46, COLOR_WHITE, colour_prefs.bkgnd);
-    init_pair(47, COLOR_GREEN, colour_prefs.bkgnd);
-    init_pair(48, COLOR_RED, colour_prefs.bkgnd);
-    init_pair(49, COLOR_YELLOW, colour_prefs.bkgnd);
-    init_pair(50, COLOR_BLUE, colour_prefs.bkgnd);
-    init_pair(51, COLOR_CYAN, colour_prefs.bkgnd);
-    init_pair(52, COLOR_BLACK, colour_prefs.bkgnd);
-    init_pair(53, COLOR_MAGENTA, colour_prefs.bkgnd);
+    init_pair(47, COLOR_WHITE, colour_prefs.bkgnd);
+    init_pair(48, COLOR_GREEN, colour_prefs.bkgnd);
+    init_pair(49, COLOR_RED, colour_prefs.bkgnd);
+    init_pair(50, COLOR_YELLOW, colour_prefs.bkgnd);
+    init_pair(51, COLOR_BLUE, colour_prefs.bkgnd);
+    init_pair(52, COLOR_CYAN, colour_prefs.bkgnd);
+    init_pair(53, COLOR_BLACK, colour_prefs.bkgnd);
+    init_pair(54, COLOR_MAGENTA, colour_prefs.bkgnd);
 }
 
 static NCURSES_COLOR_T
@@ -397,6 +402,7 @@ _load_colours(void)
     _set_colour("them",                     &colour_prefs.them,                 COLOR_GREEN,    THEME_THEM);
     _set_colour("roster.header",            &colour_prefs.rosterheader,         COLOR_YELLOW,   THEME_ROSTER_HEADER);
     _set_colour("occupants.header",         &colour_prefs.occupantsheader,      COLOR_YELLOW,   THEME_OCCUPANTS_HEADER);
+    _set_colour("receipt.sent",             &colour_prefs.receiptsent,          COLOR_RED,      THEME_RECEIPT_SENT);
 }
 
 static void
@@ -425,7 +431,9 @@ _load_preferences(void)
     _set_boolean_preference("flash", PREF_FLASH);
     _set_boolean_preference("splash", PREF_SPLASH);
     _set_boolean_preference("wrap", PREF_WRAP);
+    _set_boolean_preference("wins.autotidy", PREF_WINS_AUTO_TIDY);
     _set_string_preference("time", PREF_TIME);
+    _set_string_preference("time.statusbar", PREF_TIME_STATUSBAR);
 
     _set_boolean_preference("resource.title", PREF_RESOURCE_TITLE);
     _set_boolean_preference("resource.message", PREF_RESOURCE_MESSAGE);
@@ -435,6 +443,7 @@ _load_preferences(void)
     _set_string_preference("statuses.muc", PREF_STATUSES_MUC);
 
     _set_boolean_preference("occupants", PREF_OCCUPANTS);
+    _set_boolean_preference("occupants.jid", PREF_OCCUPANTS_JID);
     if (g_key_file_has_key(theme, "ui", "occupants.size", NULL)) {
         gint occupants_size = g_key_file_get_integer(theme, "ui", "occupants.size", NULL);
         prefs_set_occupants_size(occupants_size);
@@ -443,6 +452,7 @@ _load_preferences(void)
     _set_boolean_preference("roster", PREF_ROSTER);
     _set_boolean_preference("roster.offline", PREF_ROSTER_OFFLINE);
     _set_boolean_preference("roster.resource", PREF_ROSTER_RESOURCE);
+    _set_boolean_preference("roster.empty", PREF_ROSTER_EMPTY);
     _set_string_preference("roster.by", PREF_ROSTER_BY);
     if (g_key_file_has_key(theme, "ui", "roster.size", NULL)) {
         gint roster_size = g_key_file_get_integer(theme, "ui", "roster.size", NULL);
@@ -454,7 +464,22 @@ _load_preferences(void)
     _set_boolean_preference("presence", PREF_PRESENCE);
     _set_boolean_preference("intype", PREF_INTYPE);
 
-    _set_boolean_preference("otr.warn", PREF_OTR_WARN);
+    _set_boolean_preference("enc.warn", PREF_ENC_WARN);
+
+    if (g_key_file_has_key(theme, "ui", "otr.char", NULL)) {
+        gchar *ch = g_key_file_get_string(theme, "ui", "otr.char", NULL);
+        if (ch && strlen(ch) > 0) {
+            prefs_set_otr_char(ch[0]);
+            g_free(ch);
+        }
+    }
+    if (g_key_file_has_key(theme, "ui", "pgp.char", NULL)) {
+        gchar *ch = g_key_file_get_string(theme, "ui", "pgp.char", NULL);
+        if (ch && strlen(ch) > 0) {
+            prefs_set_pgp_char(ch[0]);
+            g_free(ch);
+        }
+    }
 }
 
 static gchar *
@@ -471,9 +496,9 @@ void
 _theme_list_dir(const gchar * const dir, GSList **result)
 {
     GDir *themes = g_dir_open(dir, 0, NULL);
-    if (themes != NULL) {
+    if (themes) {
         const gchar *theme = g_dir_read_name(themes);
-        while (theme != NULL) {
+        while (theme) {
             *result = g_slist_append(*result, strdup(theme));
             theme = g_dir_read_name(themes);
         }
@@ -487,7 +512,7 @@ _theme_find(const char * const theme_name)
     GString *path = NULL;
     gchar *themes_dir = _get_themes_dir();
 
-    if (themes_dir != NULL) {
+    if (themes_dir) {
         path = g_string_new(themes_dir);
         g_free(themes_dir);
         g_string_append(path, "/");
@@ -563,41 +588,42 @@ theme_attrs(theme_item_t attrs)
     case THEME_STATUS_NEW:              result = COLOR_PAIR(24); break;
     case THEME_ME:                      result = COLOR_PAIR(25); break;
     case THEME_THEM:                    result = COLOR_PAIR(26); break;
-    case THEME_ROOMINFO:                result = COLOR_PAIR(27); break;
-    case THEME_ROOMMENTION:             result = COLOR_PAIR(28); break;
-    case THEME_ONLINE:                  result = COLOR_PAIR(29); break;
-    case THEME_OFFLINE:                 result = COLOR_PAIR(30); break;
-    case THEME_AWAY:                    result = COLOR_PAIR(31); break;
-    case THEME_CHAT:                    result = COLOR_PAIR(32); break;
-    case THEME_DND:                     result = COLOR_PAIR(33); break;
-    case THEME_XA:                      result = COLOR_PAIR(34); break;
-    case THEME_TYPING:                  result = COLOR_PAIR(35); break;
-    case THEME_GONE:                    result = COLOR_PAIR(36); break;
-    case THEME_SUBSCRIBED:              result = COLOR_PAIR(37); break;
-    case THEME_UNSUBSCRIBED:            result = COLOR_PAIR(38); break;
-    case THEME_OTR_STARTED_TRUSTED:     result = COLOR_PAIR(39); break;
-    case THEME_OTR_STARTED_UNTRUSTED:   result = COLOR_PAIR(40); break;
-    case THEME_OTR_ENDED:               result = COLOR_PAIR(41); break;
-    case THEME_OTR_TRUSTED:             result = COLOR_PAIR(42); break;
-    case THEME_OTR_UNTRUSTED:           result = COLOR_PAIR(43); break;
-    case THEME_ROSTER_HEADER:           result = COLOR_PAIR(44); break;
-    case THEME_OCCUPANTS_HEADER:        result = COLOR_PAIR(45); break;
-    case THEME_WHITE:                   result = COLOR_PAIR(46); break;
-    case THEME_WHITE_BOLD:              result = COLOR_PAIR(46); break;
-    case THEME_GREEN:                   result = COLOR_PAIR(47); break;
-    case THEME_GREEN_BOLD:              result = COLOR_PAIR(47); break;
-    case THEME_RED:                     result = COLOR_PAIR(48); break;
-    case THEME_RED_BOLD:                result = COLOR_PAIR(48); break;
-    case THEME_YELLOW:                  result = COLOR_PAIR(49); break;
-    case THEME_YELLOW_BOLD:             result = COLOR_PAIR(49); break;
-    case THEME_BLUE:                    result = COLOR_PAIR(50); break;
-    case THEME_BLUE_BOLD:               result = COLOR_PAIR(50); break;
-    case THEME_CYAN:                    result = COLOR_PAIR(51); break;
-    case THEME_CYAN_BOLD:               result = COLOR_PAIR(51); break;
-    case THEME_BLACK:                   result = COLOR_PAIR(52); break;
-    case THEME_BLACK_BOLD:              result = COLOR_PAIR(52); break;
-    case THEME_MAGENTA:                 result = COLOR_PAIR(53); break;
-    case THEME_MAGENTA_BOLD:            result = COLOR_PAIR(53); break;
+    case THEME_RECEIPT_SENT:            result = COLOR_PAIR(27); break;
+    case THEME_ROOMINFO:                result = COLOR_PAIR(28); break;
+    case THEME_ROOMMENTION:             result = COLOR_PAIR(29); break;
+    case THEME_ONLINE:                  result = COLOR_PAIR(30); break;
+    case THEME_OFFLINE:                 result = COLOR_PAIR(31); break;
+    case THEME_AWAY:                    result = COLOR_PAIR(32); break;
+    case THEME_CHAT:                    result = COLOR_PAIR(33); break;
+    case THEME_DND:                     result = COLOR_PAIR(34); break;
+    case THEME_XA:                      result = COLOR_PAIR(35); break;
+    case THEME_TYPING:                  result = COLOR_PAIR(36); break;
+    case THEME_GONE:                    result = COLOR_PAIR(37); break;
+    case THEME_SUBSCRIBED:              result = COLOR_PAIR(38); break;
+    case THEME_UNSUBSCRIBED:            result = COLOR_PAIR(39); break;
+    case THEME_OTR_STARTED_TRUSTED:     result = COLOR_PAIR(40); break;
+    case THEME_OTR_STARTED_UNTRUSTED:   result = COLOR_PAIR(41); break;
+    case THEME_OTR_ENDED:               result = COLOR_PAIR(42); break;
+    case THEME_OTR_TRUSTED:             result = COLOR_PAIR(43); break;
+    case THEME_OTR_UNTRUSTED:           result = COLOR_PAIR(44); break;
+    case THEME_ROSTER_HEADER:           result = COLOR_PAIR(45); break;
+    case THEME_OCCUPANTS_HEADER:        result = COLOR_PAIR(46); break;
+    case THEME_WHITE:                   result = COLOR_PAIR(47); break;
+    case THEME_WHITE_BOLD:              result = COLOR_PAIR(47); break;
+    case THEME_GREEN:                   result = COLOR_PAIR(48); break;
+    case THEME_GREEN_BOLD:              result = COLOR_PAIR(48); break;
+    case THEME_RED:                     result = COLOR_PAIR(49); break;
+    case THEME_RED_BOLD:                result = COLOR_PAIR(49); break;
+    case THEME_YELLOW:                  result = COLOR_PAIR(50); break;
+    case THEME_YELLOW_BOLD:             result = COLOR_PAIR(50); break;
+    case THEME_BLUE:                    result = COLOR_PAIR(51); break;
+    case THEME_BLUE_BOLD:               result = COLOR_PAIR(51); break;
+    case THEME_CYAN:                    result = COLOR_PAIR(52); break;
+    case THEME_CYAN_BOLD:               result = COLOR_PAIR(52); break;
+    case THEME_BLACK:                   result = COLOR_PAIR(53); break;
+    case THEME_BLACK_BOLD:              result = COLOR_PAIR(53); break;
+    case THEME_MAGENTA:                 result = COLOR_PAIR(54); break;
+    case THEME_MAGENTA_BOLD:            result = COLOR_PAIR(54); break;
     default:                            break;
     }
 
@@ -607,4 +633,4 @@ theme_attrs(theme_item_t attrs)
         return result;
 
     }
-}
\ No newline at end of file
+}
