@@ -80,17 +80,18 @@ jid_create(const gchar * const str)
     gchar *domain_start = trimmed;
 
 
-    if (atp != NULL) {
+    if (atp) {
         result->localpart = g_utf8_substring(trimmed, 0, g_utf8_pointer_to_offset(trimmed, atp));
         domain_start = atp + 1;
     }
 
-    if (slashp != NULL) {
+    if (slashp) {
         result->resourcepart = g_strdup(slashp + 1);
         result->domainpart = g_utf8_substring(domain_start, 0, g_utf8_pointer_to_offset(domain_start, slashp));
         char *barejidraw = g_utf8_substring(trimmed, 0, g_utf8_pointer_to_offset(trimmed, slashp));
         result->barejid = g_utf8_strdown(barejidraw, -1);
         result->fulljid = g_strdup(trimmed);
+        g_free(barejidraw);
     } else {
         result->domainpart = g_strdup(domain_start);
         result->barejid = g_utf8_strdown(trimmed, -1);
@@ -120,7 +121,7 @@ jid_create_from_bare_and_resource(const char * const room, const char * const ni
 void
 jid_destroy(Jid *jid)
 {
-    if (jid != NULL) {
+    if (jid) {
         g_free(jid->str);
         g_free(jid->localpart);
         g_free(jid->domainpart);
@@ -169,8 +170,8 @@ get_nick_from_full_jid(const char * const full_room_jid)
     char **tokens = g_strsplit(full_room_jid, "/", 0);
     char *nick_part = NULL;
 
-    if (tokens != NULL) {
-        if (tokens[0] != NULL && tokens[1] != NULL) {
+    if (tokens) {
+        if (tokens[0] && tokens[1]) {
             nick_part = strdup(tokens[1]);
         }
 
@@ -192,4 +193,4 @@ jid_fulljid_or_barejid(Jid *jid)
     } else {
         return jid->barejid;
     }
-}
\ No newline at end of file
+}
