@@ -156,7 +156,7 @@ parse_args(const char * const inp, int min, int max, gboolean *result)
         token = g_slist_next(token);
         int arg_count = 0;
 
-        while (token != NULL) {
+        while (token) {
             args[arg_count++] = strdup(token->data);
             token = g_slist_next(token);
         }
@@ -303,7 +303,7 @@ parse_args_with_freetext(const char * const inp, int min, int max, gboolean *res
         token = g_slist_next(token);
         int arg_count = 0;
 
-        while (token != NULL) {
+        while (token) {
             args[arg_count++] = strdup(token->data);
             token = g_slist_next(token);
         }
@@ -419,7 +419,7 @@ parse_options(gchar **args, gchar **opt_keys, gboolean *res)
         }
 
         // check if duplicate
-        if (g_list_find_custom(found_keys, args[curr], (GCompareFunc)g_strcmp0) != NULL) {
+        if (g_list_find_custom(found_keys, args[curr], (GCompareFunc)g_strcmp0)) {
             *res = FALSE;
             g_list_free(keys);
             return options;
@@ -450,7 +450,7 @@ parse_options(gchar **args, gchar **opt_keys, gboolean *res)
 void
 options_destroy(GHashTable *options)
 {
-    if (options != NULL) {
+    if (options) {
         g_hash_table_destroy(options);
     }
 }
