@@ -40,20 +40,31 @@
 
 #include <glib.h>
 
+typedef struct delivery_receipt_t {
+    char *id;
+    gboolean received;
+} DeliveryReceipt;
+
 typedef struct prof_buff_entry_t {
     char show_char;
+    int pad_indent;
     GDateTime *time;
     int flags;
     theme_item_t theme_item;
     char *from;
     char *message;
+    DeliveryReceipt *receipt;
 } ProfBuffEntry;
 
 typedef struct prof_buff_t *ProfBuff;
 
 ProfBuff buffer_create();
 void buffer_free(ProfBuff buffer);
-void buffer_push(ProfBuff buffer, const char show_char, GDateTime *time, int flags, theme_item_t theme_item, const char * const from, const char * const message);
+void buffer_push(ProfBuff buffer, const char show_char, int pad_indent, GDateTime *time, int flags, theme_item_t theme_item,
+    const char * const from, const char * const message, DeliveryReceipt *receipt);
 int buffer_size(ProfBuff buffer);
 ProfBuffEntry* buffer_yield_entry(ProfBuff buffer, int entry);
+gboolean buffer_mark_received(ProfBuff buffer, const char * const id);
+
+
 #endif
