@@ -48,6 +48,7 @@
 #include "ui/ui.h"
 #include "ui/statusbar.h"
 #include "ui/inputwin.h"
+#include "config/preferences.h"
 
 #define TIME_CHECK 60000000
 
@@ -94,7 +95,7 @@ create_status_bar(void)
     mvwprintw(status_bar, 0, cols - 34 + ((current - 1) * 3), bracket);
     wattroff(status_bar, bracket_attrs);
 
-    if (last_time != NULL) {
+    if (last_time) {
         g_date_time_unref(last_time);
     }
     last_time = g_date_time_new_now_local();
@@ -126,10 +127,22 @@ status_bar_resize(void)
     mvwprintw(status_bar, 0, cols - 34 + ((current - 1) * 3), bracket);
     wattroff(status_bar, bracket_attrs);
 
-    if (message != NULL) {
-        mvwprintw(status_bar, 0, 10, message);
+    if (message) {
+        char *time_pref = prefs_get_string(PREF_TIME_STATUSBAR);
+        gchar *date_fmt = g_date_time_format(last_time, time_pref);
+        assert(date_fmt != NULL);
+        size_t len = strlen(date_fmt);
+        g_free(date_fmt);
+        if (g_strcmp0(time_pref, "") != 0) {
+            /* 01234567890123456
+             *  [HH:MM]  message */
+            mvwprintw(status_bar, 0, 5 + len, message);
+        } else {
+            mvwprintw(status_bar, 0, 1, message);
+        }
+        prefs_free_string(time_pref);
     }
-    if (last_time != NULL) {
+    if (last_time) {
         g_date_time_unref(last_time);
     }
     last_time = g_date_time_new_now_local();
@@ -289,11 +302,22 @@ status_bar_print_message(const char * const msg)
 {
     werase(status_bar);
 
-    if (message != NULL) {
+    if (message) {
         free(message);
     }
     message = strdup(msg);
-    mvwprintw(status_bar, 0, 10, message);
+
+    char *time_pref = prefs_get_string(PREF_TIME_STATUSBAR);
+    gchar *date_fmt = g_date_time_format(last_time, time_pref);
+    assert(date_fmt != NULL);
+    size_t len = strlen(date_fmt);
+    g_free(date_fmt);
+    if (g_strcmp0(time_pref, "") != 0) {
+        mvwprintw(status_bar, 0, 5 + len, message);
+    } else {
+        mvwprintw(status_bar, 0, 1, message);
+    }
+    prefs_free_string(time_pref);
 
     int cols = getmaxx(stdscr);
     int bracket_attrs = theme_attrs(THEME_STATUS_BRACKET);
@@ -309,7 +333,7 @@ status_bar_print_message(const char * const msg)
 void
 status_bar_clear(void)
 {
-    if (message != NULL) {
+    if (message) {
         free(message);
         message = NULL;
     }
@@ -330,7 +354,7 @@ status_bar_clear(void)
 void
 status_bar_clear_message(void)
 {
-    if (message != NULL) {
+    if (message) {
         free(message);
         message = NULL;
     }
@@ -412,23 +436,28 @@ _mark_inactive(int num)
 static void
 _status_bar_draw(void)
 {
-    if (last_time != NULL) {
+    if (last_time) {
         g_date_time_unref(last_time);
     }
     last_time = g_date_time_new_now_local();
-    gchar *date_fmt = g_date_time_format(last_time, "%H:%M");
-    assert(date_fmt != NULL);
 
     int bracket_attrs = theme_attrs(THEME_STATUS_BRACKET);
 
-    wattron(status_bar, bracket_attrs);
-    mvwaddch(status_bar, 0, 1, '[');
-    wattroff(status_bar, bracket_attrs);
-    mvwprintw(status_bar, 0, 2, date_fmt);
-    wattron(status_bar, bracket_attrs);
-    mvwaddch(status_bar, 0, 7, ']');
-    wattroff(status_bar, bracket_attrs);
-    g_free(date_fmt);
+    char *time_pref = prefs_get_string(PREF_TIME_STATUSBAR);
+    if (g_strcmp0(time_pref, "") != 0) {
+        gchar *date_fmt = g_date_time_format(last_time, time_pref);
+        assert(date_fmt != NULL);
+        size_t len = strlen(date_fmt);
+        wattron(status_bar, bracket_attrs);
+        mvwaddch(status_bar, 0, 1, '[');
+        wattroff(status_bar, bracket_attrs);
+        mvwprintw(status_bar, 0, 2, date_fmt);
+        wattron(status_bar, bracket_attrs);
+        mvwaddch(status_bar, 0, 2 + len, ']');
+        wattroff(status_bar, bracket_attrs);
+        g_free(date_fmt);
+    }
+    prefs_free_string(time_pref);
 
     _update_win_statuses();
     wnoutrefresh(status_bar);
