@@ -42,10 +42,6 @@ void status_bar_clear(void);
 void status_bar_clear_message(void);
 void status_bar_get_password(void);
 void status_bar_print_message(const char * const msg);
-void status_bar_inactive(const int win);
-void status_bar_active(const int win);
-void status_bar_new(const int win);
-void status_bar_set_all_inactive(void);
 void status_bar_current(int i);
 
-#endif
\ No newline at end of file
+#endif
